#ifndef WVISCSOLVER_H
#define WVISCSOLVER_H

#include "dart.h"
#include "wBLRegion.h"

namespace dart
{

class DART_API ViscSolver
{
public:
  double Cdt;
  double Cdf;
  double Cdp;
  double Re;

private:
  double Minf;
  double CFL0;
  size_t nbElmUpper;
  bool stagPtMvmt;
  bool mapMesh=false;

  TimeSolver *tSolver;
  SolutionInterp *meshConverter;

public:
  std::vector<BLRegion *> bl;

  bool printOn = true;

  ViscSolver(double _Re, double _Minf, double _CFL0, unsigned int meshFactor);

  ~ViscSolver();
  void InitMeshes(size_t region, std::vector<double> locM, std::vector<double> globM);
  void SendInvBC(size_t region, std::vector<double> _Ue,
                               std::vector<double> _Me,
                               std::vector<double> _Rhoe);
  void SendExtVariables(size_t region, std::vector<double> _xxExt, std::vector<double> _deltaStarExt);

  int Run(unsigned int couplIter);
  std::vector<double> ExtractBlowingVel(size_t iRegion);
  std::vector<double> ExtractDeltaStar(size_t iRegion);
  std::vector<double> ExtractXx(size_t iRegion);
  std::vector<std::vector<double>> ExtractSolution();
  std::vector<double> Getxtr();

private:
  void CheckWaves(size_t iPoint, BLRegion *bl);
  void AverageTransition(size_t iPoint, BLRegion *bl, int forced);
  void ComputeDrag();
  void ComputeBlwVel();
  void PrintBanner();

};
} // namespace dart
#endif //WVISCSOLVER_H